unit LbSpeedButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TLbSpeedButtonLayout = (blGlyphLeft, blGlyphRight);  // Top, Bottom etc. will follow soon!

  TLbSpeedButton = class(TGraphicControl)
  private
    FAlignment: TAlignment;
    FAllowAllUp: boolean;
    FCaption: TCaption;
    FDown: boolean;
    FFlat: boolean;
    FGlyph: TBitmap;
    FGroupIndex: integer;
    FLayout: TLbSpeedButtonLayout;
    FLightColor: TColor;
    FNumGlyphs: integer;
    FShadowColor: TColor;
    FTransparent: boolean;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseExit: TNotifyEvent;

    bCursorOnButton: boolean;
  protected
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;

    procedure SetAlignment(fNew: TAlignment);
    procedure SetFlat(fNew: boolean);
    procedure SetDown(fNew: boolean);
    procedure SetCaption(const fNew: TCaption);
    procedure SetGlyph(fNew: TBitmap);
    procedure SetGroupIndex(fNew: integer);
    procedure SetLayout(fNew: TLbSpeedButtonLayout);
    procedure SetLightColor(fNew: TColor);
    procedure SetNumGlyphs(fNew: integer);
    procedure SetShadowColor(fNew: TColor);
    procedure SetTransparent(fNew: boolean);

    procedure DoMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure DoMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  published
    property Align;
    property Alignment: TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property AllowAllUp: boolean read FAllowAllUp write FAllowAllUp default false;
    property Caption: TCaption read FCaption write SetCaption;
    property Color;
    property Down: boolean read FDown write SetDown default false;
    property Enabled;
    property Flat: boolean read FFlat write SetFlat default false;
    property Font;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property GroupIndex: integer read FGroupIndex write SetGroupIndex default 0;
    property Hint;
    property Layout: TLbSpeedButtonLayout read FLayout write SetLayout default blGlyphLeft;
    property LightColor: TColor read FLightColor write SetLightColor default clWhite;
    property NumGlyphs: integer read FNumGlyphs write SetNumGlyphs default 0;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShadowColor: TColor read FShadowColor write SetShadowColor default clBlack;
    property ShowHint;
    property Transparent: boolean read FTransparent write SetTransparent default false;
    property Visible;

    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

{##############################################################################}

procedure Register;
begin
  RegisterComponents('LB', [TLbSpeedButton]);
end;

{##############################################################################}

constructor TLbSpeedButton.Create(aOwner: TComponent);
begin
   inherited;

   ControlStyle := ControlStyle + [csSetCaption];
   ControlStyle := ControlStyle - [csOpaque];

   Height := 24;
   Width := 100;

   bCursorOnButton := false;
   FLightColor := clWhite;
   FShadowColor := clBlack;

   FGlyph := TBitmap.Create;
end;

{##############################################################################}

destructor TLbSpeedButton.Destroy;
begin
   FGlyph.Free;
   inherited;
end;

{##############################################################################}

procedure TLbSpeedButton.SetAlignment(fNew: TAlignment);
begin
   FAlignment := fNew;
   if FTransparent then invalidate else paint;  // Could have generally used invalidate,
                                                // but why make it flicker if it is not
                                                // transparent?
end;

{##############################################################################}

procedure TLbSpeedButton.SetCaption(const fNew: TCaption);
begin
   FCaption := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetDown(fNew: boolean);
var
   i: integer;

begin
   if FDown = fNew then exit;

   // If grouped, set all siblings to down=false
   if GroupIndex <> 0 then
      for i := 0 to Parent.ControlCount-1 do
         if Parent.Controls[i] is TLbSpeedButton then
            if TLbSpeedButton(Parent.Controls[i]).GroupIndex = GroupIndex then
               if TLbSpeedButton(Parent.Controls[i]) <> self then
                  TLbSpeedButton(Parent.Controls[i]).Down := false;

   FDown := fNew;

   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetFlat(fNew: boolean);
begin
   FFlat := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetGlyph(fNew: TBitmap);
begin
   FGlyph.Assign(fNew);
   if fNew.Height <> 0 then FNumGlyphs := fNew.Width div fNew.Height else FNumGlyphs := 0;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetGroupIndex(fNew: integer);
begin
   FGroupIndex := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetLayout(fNew: TLbSpeedButtonLayout);
begin
   FLayout := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetLightColor(fNew: TColor);
begin
   FLightColor := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetNumGlyphs(fNew: integer);
begin
   FNumGlyphs := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetShadowColor(fNew: TColor);
begin
   FShadowColor := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.SetTransparent(fNew: boolean);
begin
   FTransparent := fNew;
   if FTransparent then invalidate else paint;
end;

{##############################################################################}

procedure TLbSpeedButton.Paint;
var
   iCaptionHeight, iCaptionWidth, iGlyphHeight, iGlyphWidth: integer;
   iCaptionTop, iGlyphTop: integer;
   iGlyphIndex: integer;
   iOffset: integer;

   procedure DrawGlyph(iDestLeft, iDestTop, iSrcLeft, iSrcTop, iWidth, iHeight: integer);  // Glyph transparent zeichnen
   var
      aPicture: TPicture;

   begin
      aPicture := TPicture.Create;
      aPicture.Bitmap.Assign(fGlyph);
      aPicture.Bitmap.Width := iWidth;
      aPicture.Bitmap.Height := iHeight;
      aPicture.Bitmap.Canvas.Draw(-iSrcLeft, -iSrcTop, FGlyph);
      aPicture.Graphic.Transparent := true;
      Canvas.Draw(iDestLeft, iDestTop, aPicture.Graphic);
      aPicture.Free;
   end;

begin
   if not (Visible or (csDesigning in ComponentState)) then exit;

   // Background
   if FTransparent then Canvas.Brush.Style := bsclear
   else
   begin
      Canvas.Brush.Color := Color;
      Canvas.Rectangle(-1, -1, Width+1, Height+1);
   end;

   // Border
   if Enabled and (not Flat or bCursorOnButton or (csDesigning in ComponentState) or FDown) then
   begin
      with Canvas do
      begin
         if FDown then Pen.Color := FShadowColor else Pen.Color := FLightColor;
         MoveTo(0, Height-1);
         LineTo(0, 0);
         LineTo(Width-1, 0);
         if FDown then Pen.Color := FLightColor else Pen.Color := FShadowColor;
         LineTo(Width-1, Height-1);
         LineTo(0, Height-1);
      end;
   end;

   // Prepare layout
   Canvas.Font := Font;
   iCaptionHeight := Canvas.TextHeight(FCaption);
   iCaptionWidth := Canvas.TextWidth(FCaption);
   iGlyphHeight := FGlyph.Height;
   if FNumGlyphs <> 0 then iGlyphWidth := FGlyph.Width div FNumGlyphs else iGlyphWidth := 0;
   iCaptionTop := (Height - iCaptionHeight) div 2;
   iGlyphTop := (Height - iGlyphHeight) div 2;
   iGlyphIndex := 0; if not Enabled then iGlyphIndex := iGlyphWidth;
   iOffset := 0; if Down then iOffset := 1;

   // Text + Glyph
   // This part can be highly optimized, I know...
   if FLayout = blGlyphLeft then
   begin
      if FAlignment = taLeftJustify then
      begin
         Canvas.TextRect(Rect(1, 1, Width-1, Height-1), 7+iGlyphWidth+iOffset, iCaptionTop+iOffset, FCaption);
         DrawGlyph(5+iOffset, iGlyphTop+iOffset, iGlyphIndex, 0, iGlyphWidth, iGlyphHeight);
      end
      else
      if FAlignment = taRightJustify then
      begin
         Canvas.TextRect(Rect(1, 1, Width-1, Height-1), Width-5-iCaptionWidth+iOffset, iCaptionTop+iOffset, FCaption);
         DrawGlyph(Width-7-iCaptionWidth-iGlyphWidth+iOffset, iGlyphTop+iOffset, iGlyphIndex, 0, iGlyphWidth, iGlyphHeight);
      end
      else
      if FAlignment = taCenter then
      begin
         Canvas.TextRect(Rect(1, 1, Width-1, Height-1), (Width-iCaptionWidth-iGlyphWidth) div 2 + iGlyphWidth + 1 + iOffset, iCaptionTop+iOffset, FCaption);
         DrawGlyph((Width-iCaptionWidth-iGlyphWidth) div 2 - 1 + iOffset, iGlyphTop+iOffset, iGlyphIndex, 0, iGlyphWidth, iGlyphHeight);
      end;
   end
   else
   if Layout = blGlyphRight then
   begin
      if FAlignment = taLeftJustify then
      begin
         Canvas.TextRect(Rect(1, 1, Width-1, Height-1), 5+iOffset, iCaptionTop+iOffset, FCaption);
         DrawGlyph(7+iCaptionWidth+iOffset, iGlyphTop+iOffset, iGlyphIndex, 0, iGlyphWidth, iGlyphHeight);
      end
      else
      if FAlignment = taRightJustify then
      begin
         Canvas.TextRect(Rect(1, 1, Width-1, Height-1), Width-7-iCaptionWidth-iGlyphWidth+iOffset, iCaptionTop+iOffset, FCaption);
         DrawGlyph(Width-5-iGlyphWidth+iOffset, iGlyphTop+iOffset, iGlyphIndex, 0, iGlyphWidth, iGlyphHeight);
      end
      else
      if FAlignment = taCenter then
      begin
         Canvas.TextRect(Rect(1, 1, Width-1, Height-1), (Width-iCaptionWidth-iGlyphWidth) div 2 - 1 + iOffset, iCaptionTop+iOffset, FCaption);
         DrawGlyph((Width-iCaptionWidth-iGlyphWidth) div 2 + iCaptionWidth + 1 + iOffset, iGlyphTop+iOffset, iGlyphIndex, 0, iGlyphWidth, iGlyphHeight);
      end;
   end;
end;

{##############################################################################}

procedure TLbSpeedButton.DoMouseEnter(var Msg: TMessage);
begin
   bCursorOnButton := true;
   if FTransparent then invalidate else paint;
   if assigned(FOnMouseEnter) then FOnMouseEnter(self);
end;

{##############################################################################}

procedure TLbSpeedButton.DoMouseLeave(var Msg: TMessage);
begin
   bCursorOnButton := false;
   if FTransparent then invalidate else paint;
   if assigned(FOnMouseExit) then FOnMouseExit(self);
end;

{##############################################################################}

procedure TLbSpeedButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if GroupIndex = 0 then Down := true
   else
   begin
      if Down then
      begin
         if FAllowAllUp then Down := false;
      end
      else
         Down := true;
   end;
end;

{##############################################################################}

procedure TLbSpeedButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   inherited;
   if GroupIndex = 0 then Down := false;
end;

{##############################################################################}

end.
